@@ -1,5 +1,8 @@
-# This file is Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
-# License: BSD
+#
+# This file is part of LiteEth.
+#
+# Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
+# SPDX-License-Identifier: BSD-2-Clause
 
 """
 Etherbone
@@ -203,8 +206,7 @@ class LiteEthEtherboneRecordReceiver(Module):
         # # #
 
         # TODO: optimize ressources (no need to store parameters as datas)
-        fifo = stream.SyncFIFO(eth_etherbone_record_description(32), buffer_depth,
-                               buffered=True)
+        fifo = stream.SyncFIFO(eth_etherbone_record_description(32), buffer_depth, buffered=True)
         self.submodules += fifo
         self.comb += sink.connect(fifo.sink)
 
@@ -212,20 +214,12 @@ class LiteEthEtherboneRecordReceiver(Module):
         base_addr_update = Signal()
         self.sync += If(base_addr_update, base_addr.eq(fifo.source.data))
 
-        counter = Signal(max=512, reset_less=True)
-        counter_reset = Signal()
-        counter_ce = Signal()
-        self.sync += \
-            If(counter_reset,
-                counter.eq(0)
-            ).Elif(counter_ce,
-                counter.eq(counter + 1)
-            )
+        count = Signal(max=512, reset_less=True)
 
         self.submodules.fsm = fsm = FSM(reset_state="IDLE")
         fsm.act("IDLE",
             fifo.source.ready.eq(1),
-            counter_reset.eq(1),
+            NextValue(count, 0),
             If(fifo.source.valid,
                 base_addr_update.eq(1),
                 If(fifo.source.wcount,
@@ -237,15 +231,15 @@ class LiteEthEtherboneRecordReceiver(Module):
         )
         fsm.act("RECEIVE_WRITES",
             source.valid.eq(fifo.source.valid),
-            source.last.eq(counter == fifo.source.wcount-1),
+            source.last.eq(count == fifo.source.wcount-1),
             source.count.eq(fifo.source.wcount),
             source.be.eq(fifo.source.byte_enable),
-            source.addr.eq(base_addr[2:] + counter),
+            source.addr.eq(base_addr[2:] + count),
             source.we.eq(1),
             source.data.eq(fifo.source.data),
             fifo.source.ready.eq(source.ready),
             If(source.valid & source.ready,
-                counter_ce.eq(1),
+                NextValue(count, count + 1),
                 If(source.last,
                     If(fifo.source.rcount,
                         NextState("RECEIVE_BASE_RET_ADDR")
@@ -256,7 +250,7 @@ class LiteEthEtherboneRecordReceiver(Module):
             )
         )
         fsm.act("RECEIVE_BASE_RET_ADDR",
-            counter_reset.eq(1),
+            NextValue(count, 0),
             If(fifo.source.valid,
                 base_addr_update.eq(1),
                 NextState("RECEIVE_READS")
@@ -264,13 +258,13 @@ class LiteEthEtherboneRecordReceiver(Module):
         )
         fsm.act("RECEIVE_READS",
             source.valid.eq(fifo.source.valid),
-            source.last.eq(counter == fifo.source.rcount-1),
+            source.last.eq(count == fifo.source.rcount-1),
             source.count.eq(fifo.source.rcount),
             source.base_addr.eq(base_addr),
             source.addr.eq(fifo.source.data[2:]),
             fifo.source.ready.eq(source.ready),
             If(source.valid & source.ready,
-                counter_ce.eq(1),
+                NextValue(count, count + 1),
                 If(source.last,
                     NextState("IDLE")
                 )
@@ -286,12 +280,10 @@ class LiteEthEtherboneRecordSender(Module):
         # # #
 
         # TODO: optimize ressources (no need to store parameters as datas)
-        fifo = stream.SyncFIFO(eth_etherbone_mmap_description(32), buffer_depth, buffered=True)
+        fifo = PacketFIFO(eth_etherbone_mmap_description(32), buffer_depth, buffered=True)
         self.submodules += fifo
         self.comb += sink.connect(fifo.sink)
 
-        data_sel = Signal()
-
         self.submodules.fsm = fsm = FSM(reset_state="IDLE")
         fsm.act("IDLE",
             fifo.source.ready.eq(1),
@@ -300,32 +292,26 @@ class LiteEthEtherboneRecordSender(Module):
                 NextState("SEND_BASE_ADDRESS")
             )
         )
-        self.sync += [
+        self.comb += [
             source.byte_enable.eq(fifo.source.be),
             If(fifo.source.we,
                 source.wcount.eq(fifo.source.count)
             ).Else(
                 source.rcount.eq(fifo.source.count)
-            ),
-            If(data_sel,
-                source.data.eq(fifo.source.data)
-            ).Else(
-                source.data.eq(fifo.source.base_addr)
             )
         ]
-
         fsm.act("SEND_BASE_ADDRESS",
             source.valid.eq(1),
             source.last.eq(0),
+            source.data.eq(fifo.source.base_addr),
             If(source.ready,
-                data_sel.eq(1),
                 NextState("SEND_DATA")
             )
         )
         fsm.act("SEND_DATA",
             source.valid.eq(1),
             source.last.eq(fifo.source.last),
-            data_sel.eq(1),
+            source.data.eq(fifo.source.data),
             If(source.valid & source.ready,
                 fifo.source.ready.eq(1),
                 If(source.last,
@@ -336,7 +322,7 @@ class LiteEthEtherboneRecordSender(Module):
 
 
 class LiteEthEtherboneRecord(Module):
-    def __init__(self, endianness="big"):
+    def __init__(self, endianness="big", buffer_depth=4):
         self.sink   = sink   = stream.Endpoint(eth_etherbone_packet_user_description(32))
         self.source = source = stream.Endpoint(eth_etherbone_packet_user_description(32))
 
@@ -344,7 +330,7 @@ class LiteEthEtherboneRecord(Module):
 
         # Receive record, decode it and generate mmap stream
         self.submodules.depacketizer = depacketizer = LiteEthEtherboneRecordDepacketizer()
-        self.submodules.receiver = receiver = LiteEthEtherboneRecordReceiver()
+        self.submodules.receiver = receiver = LiteEthEtherboneRecordReceiver(buffer_depth)
         self.comb += [
             sink.connect(depacketizer.sink),
             depacketizer.source.connect(receiver.sink)
@@ -365,7 +351,7 @@ class LiteEthEtherboneRecord(Module):
         ]
 
         # Receive MMAP stream, encode it and send records
-        self.submodules.sender     = sender     = LiteEthEtherboneRecordSender()
+        self.submodules.sender     = sender     = LiteEthEtherboneRecordSender(buffer_depth)
         self.submodules.packetizer = packetizer = LiteEthEtherboneRecordPacketizer()
         self.comb += [
             sender.source.connect(packetizer.sink),
@@ -508,13 +494,13 @@ class LiteEthEtherboneWishboneSlave(Module):
 # Etherbone ----------------------------------------------------------------------------------------
 
 class LiteEthEtherbone(Module):
-    def __init__(self, udp, udp_port, mode="master", cd="sys"):
+    def __init__(self, udp, udp_port, mode="master", buffer_depth=4, cd="sys"):
         # Encode/encode etherbone packets
         self.submodules.packet = packet = LiteEthEtherbonePacket(udp, udp_port, cd)
 
         # Packets can be probe (etherbone discovering) or records with writes and reads
         self.submodules.probe  = probe = LiteEthEtherboneProbe()
-        self.submodules.record = record = LiteEthEtherboneRecord()
+        self.submodules.record = record = LiteEthEtherboneRecord(buffer_depth=buffer_depth)
 
         # Arbitrate/dispatch probe/records packets
         dispatcher = Dispatcher(packet.source, [probe.sink, record.sink])
